#include <stdlib.h>

#include "memory_map.h"
#include "memory_utils.h"



/* Global memory allocator. */

Allocator allocator;



/* Region management. */

void MemoryMap::insert(Region *region)
{
    std::lock_guard<std::mutex> guard(_lock);

    _regions.insert(_MemoryMapEntry(region->start, region));
}

void MemoryMap::remove(Region *region)
{
    std::lock_guard<std::mutex> guard(_lock);

    _MemoryMap::iterator entry = _regions.find(region->start);

    if (entry != _regions.end())
        _regions.erase(entry);
}

/* Show the memory state. */

void MemoryMap::show(std::ostringstream &buffer)
{
    std::lock_guard<std::mutex> guard(_lock);

    _MemoryMap::iterator it;

    buffer << "Mem: ";

    for (it = _regions.begin(); it != _regions.end(); it++)
    {
        it->second->state.show(buffer);
        buffer << " ";
    }

    buffer << std::endl;
}



/* Allocate a block of the given 'size'. */

Region *Allocator::region(offset_t size)
{
    /* Attempt to allocate aligned memory. */

    void *current;

    /* Make the size appropriate for the invocation. */

    size = round_multiple(size, PAGE_SIZE);

    if (posix_memalign(&current, size, size))
        return NULL;

    Region *region = new Region((offset_t) current, (offset_t) current + size);

    insert(region);
    return region;
}

/* Release the allocated 'region'. */

void Allocator::release(Region *region)
{
    remove(region);
    delete region;
}

// vim: tabstop=4 expandtab shiftwidth=4
